/*
 * Name:     CActor.cpp
 *
 *
 * Class:    CActor
 * Header:   CActor.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "wx/prec.h"
#include "CActor.h"

/*
 * Name :        CActor::CActor()
 * Description : Actor class constructor
 */
CActor::CActor()
{
}

/*
 * Name :        CActor::CActor()
 * Description : Constructor for an actor.
 */
CActor::CActor(const std::wstring &name) : mName(name)
{
    mEnabled = true;
    mPosition = wxPoint(0, 0);
    mClickable = true;
    mDocument = NULL;
    mRoot = NULL;
}

/*
 * Name :        CActor::~CActor()
 * Description : Actor class destructor.
 */
CActor::~CActor()
{
	// Move along now, nothing to see here
	delete mRoot;
}

/*
 * Name :        CActor::AddDrawable()
 * Description : Add a drawable to the actor. We add in order we will draw.
 * Parameters :  drawable - The drawable to add.
 * Returns :     none
 */
void CActor::AddDrawable(CDrawable *drawable)
{
	mDrawablesInOrder.push_back(drawable);
	drawable->SetActor(this);
}

/*
 * Name :        CActor::Draw()
 * Description : Draw the actor.
 * Parameters :  dc - The device context to draw on .
 * Returns :     none
 */
void CActor::Draw(wxDC *dc)
{
    // Don't draw if not enabled
    if(!mEnabled)
        return;

    // This takes care of determining the absolute placement
    // of all of the child drawables. We have to determine this
    // in tree order, which may not be the order we draw.
    if(mRoot)
        mRoot->Place(mPosition, 0);

    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->Draw(dc);
    }
}


/*
 * Name :        CActor::HitTest()
 * Description : Determine if the mouse clicked on this actor.
 * Parameters :  pos - The x,y position to test.
 * Returns :     Pointer to any drawable that was hit or NULL if none.
 */
CDrawable *CActor::HitTest(wxPoint pos)
{
    // If not enabled or not clickable, we indicate no hit.
    if(!mClickable || !mEnabled)
        return NULL;

    // Since this list is in drawing order, we realy want to know the last thing drawn
    // under the mouse, since it will be on top. So, we reverse iterate over the list.
    for(std::list<CDrawable *>::reverse_iterator d=mDrawablesInOrder.rbegin(); d!=mDrawablesInOrder.rend(); d++)
    {
        CDrawable *drawable = *d;
        if(drawable->HitTest(pos))
            return *d;
    }

    return NULL;
}
