/*
 * Name:     CDrawable.cpp
 *
 *
 * Class:    CDrawable
 * Header:   CDrawable.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "wx/prec.h"
#include "CDrawable.h"
#include <cmath>

/*
 * Name :        CDrawable::CDrawable()
 * Description : Drawable class constructor
 */
CDrawable::CDrawable()
{
}

/*
 * Name :        CDrawable::CDrawable()
 * Description : Constructor for a drawable.
 * Parameters :  name - the name of the drawable.
 * Returns :     none
 */
CDrawable::CDrawable(const std::wstring &name) : mName(name)
{
    mPosition = wxPoint(0, 0);
    mRotation = 0;
    mPlacedPosition = wxPoint(0, 0);
    mPlacedR = 0;
    mActor = NULL;
    mParent = NULL;
}

/*
 * Name :        CDrawable::!CDrawable()
 * Description : Drawable class destructor
 */
CDrawable::~CDrawable()
{
	for(std::list<CDrawable *>::iterator c=mChildren.begin(); c!=mChildren.end(); c++)
	{
		delete *c;
	}

	mChildren.clear();
}

/*
 * Name :        CDrawable::Place()
 * Description : Place the drawables at the correct location. This works heirarchically
 *               from top item down.
 * Parameters :  offset - An x,y offset
 *               rotation - A rotation
 * Returns :     none
 */
void CDrawable::Place(wxPoint offset, double rotate)
{
    // Combine the transformation we are given with the transformation
    // for this object.
    mPlacedPosition = offset + RotatePoint(mPosition, rotate);
    mPlacedR = mRotation + rotate;

    // Update our children
    for(std::list<CDrawable *>::iterator d=mChildren.begin(); d!=mChildren.end(); d++)
    {
        (*d)->Place(mPlacedPosition, mPlacedR);
    }
}

/*
 * Name :        CDrawable::AddChild()
 * Description : Add a child to his drawable's composite pattern.
 * Parameters :  child - The child to add.
 * Returns :     none
 */
void CDrawable::AddChild(CDrawable *child)
{
	mChildren.push_back(child);
	child->SetParent(this);
}

/*
 * Name :        CDrawable::IsMovable()
 * Description : Determine if a drawable is moveable. By default most are not movable.
 * Parameters :  none
 * Returns :     true if object is movable.
 */
bool CDrawable::IsMovable()
{
    return false;
}

/*
 * Name :        CDrawable::Move()
 * Description : Move the object relative to it's parent by a delta in screen coordinates.
 * Parameters :  delta - Distance to move.
 * Returns :     none
 */
void CDrawable::Move(wxPoint delta)
{
	SetPosition(wxPoint(GetPosition().x+delta.x, GetPosition().y+delta.y));
}

/*
 * Name :        CDrawable::RotatePoint()
 * Description : Rotate a point by a given angle.
 * Parameters :  point - The point to rotate
 *               angle - An angle in radians
 * Returns :     Rotated point
 *
 * Note:         This functions such that moving the cursor
 *               up rotates clockwise, and down counter-clockwise
 */
wxPoint CDrawable::RotatePoint(wxPoint point, double angle)
{
    double cosA = cos(angle);
    double sinA = sin(angle);

    return wxPoint(cosA * point.x + sinA * point.y,
                  -sinA * point.x + cosA * point.y);
}
