/*
 * Name:     CFrame.cpp
 * This is the main program frame. It owns the menus and all child windows.
 * In this application it also owns the document.
 *
 * Class:    CFrame
 * Header:   CFrame.h
 * Created : Feb 20, 2010
 * Author :  Charles B. Owen - Initial distributed version
 * Co-Author: Kenji Aono
 */

#include "wx/prec.h"
#include "CFrame.h"
#include "CDocument.h"
#include "CDocumentFactory.h"
#include "CViewActors.h"

BEGIN_EVENT_TABLE(CFrame, wxFrame)
    EVT_MENU(ID_Exit, CFrame::OnExit)
    EVT_MENU(ID_About, CFrame::OnAbout)
    EVT_MENU(ID_Expand, CFrame::OnExpand)
    EVT_MENU(ID_Move, CFrame::OnMove)
    EVT_MENU(ID_Rotate, CFrame::OnRotate)
    EVT_MENU(ID_Keyframe, CFrame::OnKeyframeSet)
    EVT_MENU(ID_NoKeyframe, CFrame::OnKeyframeClear)
END_EVENT_TABLE()

/*
 * Name :        CFrame::CFrame()
 * Description : Constructor for our frame.
 */
CFrame::CFrame() : wxFrame( NULL, -1, L"The Canadian Experience",
                            wxPoint(100, 100), wxDefaultSize)
{
    // Create our document
    CDocumentFactory factory;
    mDocument = factory.Create();

    // This is the default mode for the toolbar
    mMode = ID_Move;

    //
    // File menu
    //

    wxMenu *menuFile = new wxMenu;
    wxMenu *viewFile = new wxMenu;

    menuFile->Append(ID_About, L"&About");
    menuFile->AppendSeparator();
    wxMenuItem *exitItem = menuFile->Append( ID_Exit, L"E&xit" );
    exitItem->SetHelp(L"Exit the program");

    wxMenuItem *expandItem = viewFile->AppendCheckItem(ID_Expand, L"&Expand ");
    expandItem->SetHelp(L"Display the objects of an actor");

    //
    // Menu Bar
    //

    wxMenuBar *menuBar = new wxMenuBar;
    menuBar->Append( menuFile, L"&File");
    menuBar->Append( viewFile, L"&View");

    SetMenuBar( menuBar );

    //
    // Create a status bar
    //

    CreateStatusBar();
    SetStatusText(L"");

    //
    // Create a toolbar
    //

    CreateToolBar(wxHORIZONTAL, ID_ToolBar);
    mToolBar = GetToolBar();

    mToolBar->SetMargins(0, 0);
    AddToToolbar(mToolBar, true, L"images/tb_move.png", ID_Move, L"Move");
    AddToToolbar(mToolBar, true, L"images/tb_rotate.png", ID_Rotate, L"Rotate");
    AddToToolbar(mToolBar, false, L"images/tb_keyframe.png", ID_Keyframe, L"Set Keyframe");
    AddToToolbar(mToolBar, false, L"images/tb_nokeyframe.png", ID_NoKeyframe, L"Clear Keyframe");

    //
    // Create the child windows
    //

    mViewTimeline = new CViewTimeline(this);
    mViewEdit = new CViewEdit(this);
    mViewActors = new CViewActors(this);

    //
    // Create the sizers
    //

    // A vertical sizer so we an put two windows on top of each other
    wxBoxSizer *vertSizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *horzSizer = new wxBoxSizer(wxHORIZONTAL);

    horzSizer->Add(mViewActors, 0, wxEXPAND | wxALL, 0);   // Add actors view as first in horzSizer
    horzSizer->Add(mViewEdit, 1, wxEXPAND | wxALL, 0);     // Add edit view as second in horzSizer

    vertSizer->Add(horzSizer, 1, wxEXPAND | wxALL);        // horzSizer is first (top) in vertSizer
    vertSizer->Add(mViewTimeline, 0, wxEXPAND | wxALL, 0); // Add timeline view as second in vertSizer

    SetSizer(vertSizer);
    vertSizer->Fit(this);
    mViewEdit->SetMinSize(wxSize(100, 100));

    SetInterface();
}

/*
 * Name :        CFrame::~CFrame()
 * Description : CFrame destructor. Destroys anything this class owns.
 */
CFrame::~CFrame()
{
	// Cleaning up
	delete mDocument;
}



/*
 * Name :        CFrame::AddToToolbar()
 * Description : Add a bitmap to the toolbar.
 * Parameters :  toolBar - the toolbar we are adding to.
 *               checked - true if we use a checked item (stays pressed).
 *               id - An id for this toolbar button
 *               name - Associated tool name for tooltip.
 * Returns :     None
 */
void CFrame::AddToToolbar(wxToolBar *toolBar, bool checked, const std::wstring &file, int id, const std::wstring &name)
{
    // It is okay to create this as a local variable, because the
    // system actually copies the bitmap to the toolbar, so it's not
    // referred to later.
    wxBitmap bitmap(file.c_str());
    if(checked)
    {
        toolBar->AddCheckTool(id, L"", bitmap, wxNullBitmap, name.c_str());
    }
    else
    {
        toolBar->AddTool(id, L"", bitmap, name.c_str());
    }
}



/*
 * Name :        CFrame::SetInterface()
 * Description : Set user interface features
 * Parameters :  none
 * Returns :     none
 */
void CFrame::SetInterface()
{
    mToolBar->ToggleTool(ID_Move, mMode == ID_Move);
    mToolBar->ToggleTool(ID_Rotate, mMode == ID_Rotate);
}


/*
 * Name :        CFrame::OnMove()
 * Description : Handle a press of the Move button on the toolbar
 * Parameters :  event - The command event
 * Returns :     none
 */
void CFrame::OnMove(wxCommandEvent& event)
{
    mMode = ID_Move;
    SetInterface();
}


/*
 * Name :        CFrame::OnRotate()
 * Description : Handle a press of the rotate button on the toolbar
 * Parameters :  event - The command event
 * Returns :     none
 */
void CFrame::OnRotate(wxCommandEvent& event)
{
    mMode = ID_Rotate;
    SetInterface();
}


/*
 * Name :        CFrame::OnKeyframeSet()
 * Description : Handle a press of the keyframe set button on the toolbar
 * Parameters :  event - the command event
 * Returns :     none
 */
void CFrame::OnKeyframeSet(wxCommandEvent& event)
{

}

/*
 * Name :        CFrame::OnKeyframeClear()
 * Description : Handle a press of the keyframe clear button on the toolbar
 * Parameters :  event - the command event
 * Returns :     none
 */
void CFrame::OnKeyframeClear(wxCommandEvent& event)
{

}


/*
 * Name :        CFrame::OnExit()
 * Description : This function is called when an exit menu item is selected.
 *               It closes the main window, which ends the program.
 * Parameters :  event - An object that describes the event.
 */
void CFrame::OnExit(wxCommandEvent& event)
{
    Close(TRUE);
}


/*
 * Name :        CFrame::OnAbout()
 * Description : Menu handler for File/About menu option
 * Parameters :  event - The menu event
 * Returns :     none
 */
void CFrame::OnAbout(wxCommandEvent& event)
{
    wxMessageBox(L"The Canadian Experience is a Doc Owen Production",
                 L"About the Canadian Experience",
                  wxOK | wxICON_INFORMATION, this);
}

/*
 * Name :        CFrame::OnAbout()
 * Description : Menu handler for File/About menu option
 * Parameters :  event - The menu event
 * Returns :     none
 */
void CFrame::OnExpand(wxCommandEvent& event)
{
	mViewActors->TogDisp();
	mViewActors->UpdateObserver();
}
